#!/usr/bin/env node
'use strict';

var path = require('path');
var dotenv = require('dotenv');
var findUp = require('find-up');
var util = require('util');
var _ = require('ramda');
var _$1 = require('ramda-adjunct');
require('lodash.merge');
var keyBy = require('lodash.keyby');
var every = require('lodash.every');
var _$2 = require('chalk');
var TS = require('trie-search');
var parse$1 = require('minimist');
var RenderKid = require('renderkid');
require('singleline');
var lodash = require('lodash');
var mapValues = require('lodash.mapvalues');
var toCamelCase = require('to-camel-case');
var prettyjson = require('prettyjson');
require('loud-rejection/register');
var luabundle = require('luabundle');
var fs = require('fs');
var globOg = require('glob');
var promisify = require('es6-promisify');
var jsonFile = require('jsonfile');
var editor = require('edit-json-file');
var fsExtra = require('fs-extra');
var rollup$1 = require('rollup');
var pick = require('lodash.pick');
var vuePlugin = require('rollup-plugin-vue');
var commonjs = require('rollup-plugin-commonjs');
var globals = require('rollup-plugin-node-globals');
var fileExists = require('file-exists');
var uglifyEs = require('uglify-es');
var babel = require('rollup-plugin-babel');
var uglify = require('rollup-plugin-uglify');
var filesize = require('rollup-plugin-filesize');
var markdown = require('rollup-plugin-markdown');
var intersection = require('lodash.intersection');
var nodeResolve = require('rollup-plugin-node-resolve');
var postcss = require('rollup-plugin-postcss');
var map = require('lodash.map');
var partition = require('lodash.partition');
var match = require('minimatch');
var frontMatter = require('front-matter');
var editor$1 = require('external-editor');
var exec = require('rollup-plugin-executable-output');
var dedent = require('dedent');
var inquirer = require('inquirer');
var semver = require('semver');
var has = require('lodash.has');
var partial = require('lodash.partial');
var configureGit = require('simple-git/promise');
var uniq = require('lodash.uniq');
var detective$1 = require('detective-es6');
var execa = require('execa');
var registryUrl = require('registry-url');
var localOrHomeNpmrc = require('local-or-home-npmrc');
var filter$2 = require('lodash.filter');
var fuzzy = require('fuzzy');
var autocomplete = require('inquirer-autocomplete-prompt');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var findUp__default = /*#__PURE__*/_interopDefaultLegacy(findUp);
var keyBy__default = /*#__PURE__*/_interopDefaultLegacy(keyBy);
var every__default = /*#__PURE__*/_interopDefaultLegacy(every);
var ___default = /*#__PURE__*/_interopDefaultLegacy(_$2);
var TS__default = /*#__PURE__*/_interopDefaultLegacy(TS);
var parse__default = /*#__PURE__*/_interopDefaultLegacy(parse$1);
var RenderKid__default = /*#__PURE__*/_interopDefaultLegacy(RenderKid);
var mapValues__default = /*#__PURE__*/_interopDefaultLegacy(mapValues);
var toCamelCase__default = /*#__PURE__*/_interopDefaultLegacy(toCamelCase);
var globOg__default = /*#__PURE__*/_interopDefaultLegacy(globOg);
var promisify__default = /*#__PURE__*/_interopDefaultLegacy(promisify);
var jsonFile__default = /*#__PURE__*/_interopDefaultLegacy(jsonFile);
var editor__default = /*#__PURE__*/_interopDefaultLegacy(editor);
var pick__default = /*#__PURE__*/_interopDefaultLegacy(pick);
var vuePlugin__default = /*#__PURE__*/_interopDefaultLegacy(vuePlugin);
var commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
var globals__default = /*#__PURE__*/_interopDefaultLegacy(globals);
var babel__default = /*#__PURE__*/_interopDefaultLegacy(babel);
var uglify__default = /*#__PURE__*/_interopDefaultLegacy(uglify);
var filesize__default = /*#__PURE__*/_interopDefaultLegacy(filesize);
var markdown__default = /*#__PURE__*/_interopDefaultLegacy(markdown);
var intersection__default = /*#__PURE__*/_interopDefaultLegacy(intersection);
var nodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(nodeResolve);
var postcss__default = /*#__PURE__*/_interopDefaultLegacy(postcss);
var map__default = /*#__PURE__*/_interopDefaultLegacy(map);
var partition__default = /*#__PURE__*/_interopDefaultLegacy(partition);
var match__default = /*#__PURE__*/_interopDefaultLegacy(match);
var frontMatter__default = /*#__PURE__*/_interopDefaultLegacy(frontMatter);
var editor__default$1 = /*#__PURE__*/_interopDefaultLegacy(editor$1);
var exec__default = /*#__PURE__*/_interopDefaultLegacy(exec);
var dedent__default = /*#__PURE__*/_interopDefaultLegacy(dedent);
var inquirer__default = /*#__PURE__*/_interopDefaultLegacy(inquirer);
var has__default = /*#__PURE__*/_interopDefaultLegacy(has);
var partial__default = /*#__PURE__*/_interopDefaultLegacy(partial);
var configureGit__default = /*#__PURE__*/_interopDefaultLegacy(configureGit);
var uniq__default = /*#__PURE__*/_interopDefaultLegacy(uniq);
var detective__default = /*#__PURE__*/_interopDefaultLegacy(detective$1);
var execa__default = /*#__PURE__*/_interopDefaultLegacy(execa);
var registryUrl__default = /*#__PURE__*/_interopDefaultLegacy(registryUrl);
var localOrHomeNpmrc__default = /*#__PURE__*/_interopDefaultLegacy(localOrHomeNpmrc);
var filter__default = /*#__PURE__*/_interopDefaultLegacy(filter$2);
var autocomplete__default = /*#__PURE__*/_interopDefaultLegacy(autocomplete);

const sync = ({ anchor = '.git' } = {}) => path.dirname(findUp__default['default'].sync(anchor));

var findRepoRoot = (({ anchor = '.git' } = {}) => findUp__default['default'](anchor).then(path.dirname).catch(() => process.cwd()));

dotenv.config({ path: path.resolve(sync(), '.env') });

_.flip(_.concat);
const toList = _.pipe(_$1.list, _$1.flattenDepth(1));
_.map(toList);

_.curry((char, str) => _.pipe(_$1.trimCharsStart(char), _$1.trimCharsEnd(char))(str));

const prop = _.useWith(_.lensPath, [_.split('.')]);

const getProp = _.curryN(2, (key = '', value = {}) => {
    if (!_$1.isString(key) || !_$1.isPlainObject(value)) return value;

    return _.pipe(_.useWith(_.view, [prop]), _.defaultTo(value))(key, value);
});

const setProp = _.useWith(_.set, [prop]);

const $ = obj => ({
    pipe: operators => _.reduce(_.applyTo, obj, operators)
});

/* eslint-disable no-console */

const log = (..._) => {
    console.log(..._);
    return _[0];
};

const err = (..._) => {
    console.error(..._);
    return _[0];
};

const deep = (...args) => {
    const inspectOpts = {
        showHidden: false,
        depth: null,
        colors: true,
        maxArrayLength: null
    };
    const toPrint = _.intersperse(' ', args);
    const inspectOrNot = _ => _$1.isPlainObject(_) ? util.inspect(_, inspectOpts) : _;
    const formatted = _.map(inspectOrNot, toPrint);

    _.forEach(formatted, _ => process.stdout.write(_));
    console.log();

    return args[0];
};

const note = (...messages) => _ => {
    log(...messages);
    return _;
};
const label = (...messages) => _ => {
    log(...messages, _);
    return _;
};

log.err = err;
log.deep = deep;
log.note = note;
log.label = label;

// eslint-disable-next-line complexity
var sanitizer = (commands => {
    if (!Array.isArray(commands)) throw new Error('Commands must be an array');
    if (commands.length <= 0) throw new Error('No commands defined');
    if (!every__default['default'](commands, command => typeof command.run === 'function')) throw new Error('All commands must have a run method');
    if (!every__default['default'](commands, command => 'name' in command)) throw new Error('Not every command has a name');
});

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toArray = function (arr) {
  return Array.isArray(arr) ? arr : Array.from(arr);
};

var createSearcher = (({ commands = [] } = {}) => {
    const parsedCommands = commands.map(command => {
        const aliases = Array.isArray(command.aliases) ? command.aliases : [command.aliases];

        return _extends({}, command, {
            tokens: [command.name, ...aliases].join(' ')
        });
    });
    const searcher = new TS__default['default'](['tokens']);
    searcher.addAll(parsedCommands);
    return searcher;
});

var createDecider = (({ proposedDefault, commands, log }) => {
    const commandArray = Object.values(commands);
    const onlyCommand = commandArray.length === 1 ? commandArray[0] : null;

    const resolvedDefault = typeof proposedDefault === 'string' ? commands[proposedDefault] : proposedDefault;

    const defaultCommand = resolvedDefault || onlyCommand;
    const searcher = createSearcher({ commands: commandArray });

    return request => {
        if (!request) return defaultCommand;

        const results = searcher.get(request);
        if (results.length > 0) return results[0];

        if (defaultCommand) {
            log();
            log(_$2.yellow(` ► No command associated with \`${request}\``));
            log(_$2.yellow(` ► Running default command \`${defaultCommand.name}\``));
            log();
            return defaultCommand;
        }

        log(_$2.red(` ► No command associated with \`${request}\``));
        return process.exit(127);
    };
});

/* eslint-disable import/prefer-default-export, no-unused-vars */

const noPlug = (name, fn) => {
    const $ = `The [${name}] tool does not support a [${fn}] plug.`;
    return ___default['default'].red($);
};

// eslint-disable-next-line complexity
var log$1 = ((title, path, logger = log) => value => {
    if (!title) {
        logger(value);
        return Promise.resolve(value);
    }
    if (typeof title === 'function') {
        title(value);
        return Promise.resolve(value);
    }
    if (title && !path) {
        if (lodash.has(value, title)) {
            logger(getProp(title, value));
            return Promise.resolve(value);
        }
        logger(title, value);
        return Promise.resolve(value);
    }
    if (title && typeof path === 'function') {
        if (lodash.has(value, title)) {
            path(getProp(title, value));
            return Promise.resolve(value);
        }
        path(title, value);
        return Promise.resolve(value);
    }
    if (title && path) {
        if (lodash.has(value, path)) {
            logger(title, getProp(path, value));
            return Promise.resolve(value);
        }
        logger(title, value);
        return Promise.resolve(value);
    }
    return Promise.resolve(value);
});

// eslint-disable-next-line complexity
const merger = (obj, value) => {
    if (!value) return obj;
    if (!obj) return value;
    if (Array.isArray(obj) && Array.isArray(value)) return [...obj, ...value];
    if (Array.isArray(obj) && !Array.isArray(value)) return [...obj, value];
    if (_$1.isPlainObject(obj) && !_$1.isPlainObject(value)) return obj;
    if (_$1.isPlainObject(obj) && Array.isArray(value)) return obj;
    if (_$1.isPlainObject(obj) && _$1.isPlainObject(value)) return _.mergeWith(merger, obj, value);

    return obj;
};

// eslint-disable-next-line complexity
var extend = ((path, obj) => snowball => {
    const value = _$1.isPlainObject(snowball) ? _extends({}, snowball) : snowball;
    // eslint-disable-next-line no-nested-ternary
    const paths = obj && Array.isArray(path) ? path : !path ? null : [path];
    const finalPath = obj ? paths : null;
    const base = obj || path;
    let finalObject = null;
    try {
        finalObject = typeof base === 'function' ? base(value) : base;
    } catch (e) {
        e.value = value;
        throw e;
    }
    if (finalPath) {
        return Promise.all(finalPath.map(single => Promise.resolve(finalObject).then(o => setProp(single.toString(), o, {})).then(o => merger(value, o)))).then(o => o.reduce(merger, {})).catch(e => {
            e.value = value;
            throw e;
        });
    }
    return Promise.resolve(finalObject).then(o => merger(value, o)).catch(e => {
        e.value = value;
        throw e;
    });
});

var extendEach = ((...args) => value => {
    var _$cond = _.cond([[_.equals(2), _.always([undefined, args[0], args[1]])], [_.equals(3), _.always(args)]])(args.filter(_.identity).length),
        _$cond2 = slicedToArray(_$cond, 3);

    const key = _$cond2[0],
          target = _$cond2[1],
          map = _$cond2[2];


    return $(value).pipe([getProp(target), _.ifElse(_$1.isPlainObject, _.mapObjIndexed((v, k) => map(v, value, k)), _$1.mapIndexed((v, k) => map(v, value, k))), _$1.list, _$1.flattenDepth(1), _.cond([[_.pipe(_.head, _$1.isPlainObject), _.reduce(merger, {})], [_.pipe(_.head, _$1.isArray), _.concat([])], [_.pipe(_.head, _.identity), _.identity]]), v => extend(key, v)(value)]);
});

var under = (path => value => Promise.resolve(setProp(path, value, {})));

var ifOperator = ((path, transducer) => value => {
    if (Array.isArray(path)) {
        if (Array.every(path, p => lodash.has(value, p))) return transducer(value);
        return Promise.resolve(value);
    }
    if (lodash.has(value, path)) return transducer(value);
    return Promise.resolve(value);
});

// eslint-disable-next-line complexity
var doOperator = ((path, sideEffect) => value => {
    const finalPath = sideEffect ? path : null;
    const finalSideEffect = sideEffect || path;
    const finalValue = finalPath ? getProp(finalPath, value) : value;
    let result = finalSideEffect;
    if (typeof finalSideEffect === 'function') result = finalSideEffect(finalValue);
    return Promise.resolve(result).then(() => value);
});

var ifNo = ((path, transducer) => value => {
    if (Array.isArray(path)) {
        if (Array.every(path, p => !lodash.has(value, p))) return transducer(value);
        return Promise.resolve(value);
    }
    if (!lodash.has(value, path)) return transducer(value);
    return Promise.resolve(value);
});

var switchOperator = ((path, branches) => value => {
    const test = getProp(path, value);
    if (lodash.has(branches, test)) return branches[test](value);
    // eslint-disable-next-line dot-notation
    if (lodash.has(branches, '_default')) return branches['_default'](value);
    return Promise.resolve(value);
});

var doEach = ((iterable, stream) => value => Promise.all(getProp(iterable, value).map(iterator => stream(value, iterator))).then(results => results.reduce((v, result) => doOperator(v)(result), value)));

var operators = {
    extend,
    extendEach,
    do: doOperator,
    doEach,
    if: ifOperator,
    ifNo,
    log: log$1,
    switch: switchOperator,
    under
};

var isolator = ((...args) => {
    const fn = path => value => {
        if (!Array.isArray(path)) return getProp(path, value);
        if (path.length === 0) return value;

        return path.reduce((result, currentPath) => {
            if (lodash.has(value, currentPath)) {
                return merger(result, setProp(currentPath, getProp(currentPath, value), {}));
            }
            return result;
        }, {});
    };
    if (args.length === 1) return fn(args[0]);
    return fn(args[0])(args[1]);
});

const createApi = ops => transducers => value => {
    if (value === undefined) {
        const pipeline = val => ({
            then: callback => transducers.reduce((prev, next) => prev.then(next), Promise.resolve(val)).then(callback)
        });
        return Object.assign(pipeline, mapValues__default['default'](ops, operator => (...args) => {
            transducers.push(operator(...args));
            return pipeline;
        }));
    }

    const api = mapValues__default['default'](ops, operator => (...args) => {
        transducers.push(operator(...args));
        return api;
    });
    api.then = callback => transducers.reduce((prev, next) => prev.then(next), Promise.resolve(value)).then(callback);
    return api;
};

var configure = (ops => value => createApi(ops)([])(value));

const transmuter = configure(operators);
Object.assign(transmuter, operators);
const isolate = isolator;

var convertArgs = (({ args = {}, conversions = {} }) => Object.entries(args).map(([key, value]) => [key !== '_' ? toCamelCase__default['default'](key) : key, value]).map(([key, value]) => key in conversions ? [conversions[key], value] : [key, value]).reduce((obj, pair) => _extends({}, obj, { [pair[0]]: pair[1] }), {}));

var getKeys = ((obj = {}, path = '') => _.pipe(_.always(obj), _.compose(_.view, _.lensPath, _.split('.'), _.unless(_.is(String), _.always('')))(path), _.defaultTo(obj), _.keys)());

const prettify = _.curryN(1, prettyjson.render);
const formatValue = _.ifElse(_.is(Array), _.join('\n'), prettify);
const printer = render => _.pipe(formatValue, render);
const byType = _.indexBy(_.prop('type'));

var createExecutor = (({ config: initialConfig, renderer, style, decide } = {}) => {
    var _initialConfig$name = initialConfig.name;
    const name = _initialConfig$name === undefined ? '???' : _initialConfig$name;
    var _initialConfig$plugin = initialConfig.plugins;
    const unkeyed = _initialConfig$plugin === undefined ? [] : _initialConfig$plugin;
    var _initialConfig$plugs = initialConfig.plugs;
    const plugs = _initialConfig$plugs === undefined ? {} : _initialConfig$plugs;

    const plugins = byType(unkeyed);
    const config = _extends({}, initialConfig, {
        plugins,
        plug: fn => data => {
            // If you are debugging a problem with a third party library using KLI
            // Let the devs know they need to add the respective plug to their tool
            if (!(fn in plugs)) throw Error(noPlug(name, fn));

            const type = data.type;

            if (!type) return plugs[fn](data);

            const plugin = plugins[type] || {};
            const plug = plugin[fn] || plugs[fn];

            return plug(data);
        },
        plugAll: fn => data => {
            // If you are debugging a problem with a third party library using KLI
            // Let the devs know they need to add the respective plug to their tool
            if (!(fn in plugs)) throw Error(noPlug(name, fn));

            const allFns = _.map(_.prop(fn))(plugins);
            const fns = _.filter(_.identity, _.values(allFns));

            if (fns.length <= 0) return {};

            const results = _.map(_.applyTo(data), allFns);
            return results;
        }

        // eslint-disable-next-line complexity
    });return (unparsed = ['']) => {
        const parsed = parse__default['default'](unparsed, { boolean: true });

        var _parsed$_ = toArray(parsed._);

        const request = _parsed$_[0],
              commandArgs = _parsed$_.slice(1),
              commit = parsed.commit,
              json = parsed.json,
              dry = parsed.dry,
              keys = parsed.keys;

        const command = decide(request);
        const args = _extends({}, parsed, { _: commandArgs });
        const newArgs = convertArgs({ args, conversions: command.args });
        const finalConfig = _extends({}, config, newArgs);

        const formatter = new RenderKid__default['default']();
        formatter.style(style);
        const run = () => command.run(finalConfig);
        const template = command.template ? data => formatter.render(command.template(finalConfig)(data)) : _.identity;
        const effect = command.effect ? data => command.effect(finalConfig)(data) : _.identity;

        const result = Promise.resolve(run());

        const render = command.renderer || renderer;
        const print = printer(render);

        if (keys) return result.then(results => print(getKeys(results, keys)));
        if (json) {
            const path = () => json.indexOf(',') >= 0 ? json.split(',') : json;
            const paths = json === true ? [] : path();

            const newRun = _.pipe(run, _.andThen(isolate(paths)), _.andThen(print));
            return newRun();
        }
        return result.then(data => {
            render(template(data));

            if (commit || command.autoCommit && !dry) effect(data);
        });
    };
});

const configureKli = ({ style = {}, renderer = log } = {}) => ({
    default: proposedDefault,
    config = {},
    commands: rawCommands = []
} = {}) => {
    sanitizer(rawCommands);
    const commands = keyBy__default['default'](rawCommands, 'name');

    const decide = createDecider({ commands, proposedDefault, log: renderer });
    const execute = createExecutor({ config, style, renderer, decide });

    return { execute };
};

const configure$1 = configureKli;

var style = {
    title: {
        display: 'block',
        color: 'bright-white',
        marginLeft: 2,
        marginBottom: 1
    },
    check: {
        display: 'block',
        marginLeft: 4
    },
    details: {
        display: 'block',
        color: 'grey',
        marginLeft: 4
    },
    finer: {
        display: 'block',
        color: 'grey',
        marginLeft: 6
    },
    success: {
        display: 'block',
        color: 'grey',
        bullet: `"${_$2.green('✔')} "`
    },
    fail: {
        display: 'block',
        color: 'red',
        bullet: `"${_$2.red('✖')} "`
    },
    warning: {
        display: 'block',
        color: 'bright-yellow',
        bullet: `"${_$2.yellow('⚠')} "`
    },
    list: {
        display: 'block',
        marginTop: 1
    },
    item: {
        display: 'block',
        marginBottom: 1
    },
    bullet: {
        display: 'block',
        marginLeft: 2
    },
    change: { display: 'block' },
    type: { color: 'white' },
    scope: { color: 'bright-white' },
    subject: { color: 'grey' },
    attr: {
        display: 'block',
        marginLeft: 2
    },
    key: {
        color: 'grey',
        marginRight: 1
    },
    value: { color: 'white' },
    bar: {
        display: 'block',
        color: 'green',
        bullet: `"${_$2.green('────────────────────────────────────────────────────────────────────────────────')}"`
    },
    h1: {
        display: 'block',
        color: 'bright-white',
        marginLeft: 2
    }
};

const readFile = jsonFile__default['default'].readFile,
      writeFile = jsonFile__default['default'].writeFile,
      writeFileSync = jsonFile__default['default'].writeFileSync;


const read = promisify__default['default'](readFile);

const writeJson = promisify__default['default'](writeFile);
const write = (file, data) => writeJson(file, data, { spaces: 4 });

const writeSync = (file, data) => writeFileSync(file, data, { spaces: 4 });

const edit = (file, location, data) => {
    const json = editor__default['default'](file, { stringify_width: 4, stringify_eol: true });
    json.set(location, data);
    json.save();
};

var jsonFunctions = {
    read,
    write,
    writeSync,
    edit
};

const readStream = fs.createReadStream;
const mkdir = path => fs.mkdirSync(path, { recursive: true });
const readDir = fs.readdirSync;
const exists = fs.existsSync;
const readSync = fs.readFileSync;
const writeSync$1 = fs.writeFileSync;

const lstat = promisify__default['default'](fs.lstat);
const copy = promisify__default['default'](fsExtra.copy);
const read$1 = promisify__default['default'](fs.readFile);
const write$1 = promisify__default['default'](fs.writeFile);

const glob = promisify__default['default'](globOg__default['default']);

const json = jsonFunctions;

const some = { readStream, mkdir, readDir, exists, readSync, writeSync: writeSync$1 };
_extends({}, some, { lstat, copy, read: read$1, write: write$1, glob, json });

const type = 'lua-rock';

var luaRock = {
    type,
    details: () => ({ main: 'init.lua' }),
    manifest: ({ name }) => ({ name }),
    config: ({ structure }) => ({
        type,
        post: _extends({}, structure)
    }),
    filter: ({ files = [] }) => files.includes('init.lua'),
    build: (() => {
        var _ref = asyncToGenerator(function* ({
            config: {
                post: { root, srcDir, destDir }
            }
        }) {
            const moduleRoot = path.resolve(root, srcDir, 'init.lua');
            const nameBased = path.resolve(root, 'src/node_modules/?.lua');
            const initBased = path.resolve(root, 'src/node_modules/?/init.lua');
            const source = luabundle.bundle(moduleRoot, { paths: [nameBased, initBased] });
            const dir = path.resolve(root, destDir);

            try {
                yield lstat(dir);
            } catch (e) {
                mkdir(dir);
            }

            return write$1(path.resolve(dir, 'init.lua'), source);
        });

        return function build(_x) {
            return _ref.apply(this, arguments);
        };
    })(),
    postBuild: () => {}
};

const paths = (obj, props = [], root) => props.reduce((finalPaths, prop) => _extends({}, finalPaths, {
    [prop]: path.resolve(root, obj[prop])
}), {});

var build = (({ root, config }) => rollup$1.rollup(_extends({}, config.in, paths(config.in, ['input'], root))).then(bundle => Promise.all(config.out.map(output => bundle.write(_extends({}, output, paths(output, ['file'], root)))))));

const requiredProps = ['description', 'homepage', 'author', 'repository', 'bugs', 'license'];

var postBuild = (({ root, rootManifest, config }) => Promise.all([copy(path.resolve(root, 'LICENSE'), path.resolve(root, config.destDir, 'LICENSE')), copy(path.resolve(root, config.srcDir, 'readme.md'), path.resolve(root, config.destDir, 'readme.md')), write(path.resolve(root, config.destDir, 'package.json'), _extends({
    name: config.name
}, pick__default['default'](rootManifest, requiredProps), config.packageConfig, {
    dependencies: _extends({}, config.packageConfig.dependencies, config.dependencyVersions)
}))]));

const cache = {};

var getJsDetails = (({ root, name, structure: { srcDir } }) => {
    if (name in cache) return cache[name];

    const packageJson = path.resolve(root, srcDir, 'package.json');

    // eslint-disable-next-line global-require, import/no-dynamic-require
    const manifest = fileExists.sync(packageJson) ? require(packageJson) : {};
    cache[name] = manifest;

    return manifest;
});

var stripPartImports = ((packages = []) => packages.map(pkg => {
    if (!pkg.startsWith('@') && pkg.includes('/')) return pkg.split('/')[0];

    const numOfSlashes = (pkg.match(/\//g) || []).length;
    if (pkg.startsWith('@') && numOfSlashes > 1) {
        const split = pkg.split('/');
        return `${split[0]}/${split[1]}`;
    }

    return pkg;
}));

const babelConfig = {
    babelrc: false,
    presets: [['env', { modules: false, targets: { node: '6' } }], 'stage-0'],
    plugins: ['external-helpers'],
    exclude: ['node_modules/**']
};

const input = ({ file, dependencies, structure, plugins }) => ({
    input: path.resolve(structure.root, structure.srcDir, file),
    external: dependencies.external,
    plugins
});

const output = ({ file, structure, format, banner, exports }) => ({
    file: path.resolve(structure.root, structure.destDir, file),
    format,
    banner,
    exports
});

const postInfo = ({
    structure: { name, srcDir, destDir },
    manifest,
    dependencies: { external },
    rootManifest: { dependencies: rootDependencies = {} }
}) => ({
    name,
    srcDir,
    destDir,
    packageConfig: manifest,
    dependencyVersions: pick__default['default'](rootDependencies, intersection__default['default'](stripPartImports(external), Object.keys(rootDependencies)))
});

var rollup = (({
    type,
    file,
    outFile,
    dependencies,
    structure,
    manifest,
    rootManifest,
    format,
    banner,
    exports,
    plugins
}) => ({
    type,
    in: input({ file, dependencies, structure, plugins }),
    out: [output({ structure, file: outFile || file, format, banner, exports })],
    post: postInfo({ structure, manifest, dependencies, rootManifest })
}));

const commonPlugins = [markdown__default['default'](), nodeResolve__default['default']({
    extensions: ['.js', '.json', '.md', '.vue'],
    customResolveOptions: {
        moduleDirectory: ['src/node_modules', 'node_modules']
    }
}), babel__default['default'](babelConfig), uglify__default['default']({}, uglifyEs.minify), filesize__default['default']()];

const type$1 = 'vue-site';
const plugins = [vuePlugin__default['default']({ preprocessStyles: true }), postcss__default['default'](), commonjs__default['default'](), globals__default['default'](), ...commonPlugins];
const rollupConfig = { type: type$1, plugins, file: 'app.js', format: 'iife' };

var vueSite = {
    type: type$1,
    details: () => ({ main: 'app.js' }),
    filter: ({ files = [] }) => files.includes('index.html') && files.includes('app.js'),
    manifest: getJsDetails,
    config: snowball => [rollup(_extends({}, rollupConfig, snowball, { format: 'iife' }))],
    build,
    postBuild: ({ root, rootManifest, config }) => postBuild({ root, rootManifest, config }).then(() => copy(path.resolve(root, config.srcDir, 'index.html'), path.resolve(root, config.destDir, 'index.html')))
};

const type$2 = 'vue-component';

var vueComponent = {
    type: type$2,
    details: () => ({ main: 'index.vue' }),
    filter: ({ files = [] }) => files.includes('index.vue'),
    manifest: getJsDetails
};

var resolveName = (pkg => pkg.indexOf('/') >= 0 ? pkg.split('/')[1] : pkg);

const glob$1 = promisify__default['default'](globOg__default['default']);
const have = file => path => match__default['default'](file, path);

var findPackages = (() => {
    var _ref = asyncToGenerator(function* ({ root = '.', exclude = [], include = [] } = {}) {
        const packagesDir = path.resolve(root, 'src/node_modules');
        const stats = yield lstat(packagesDir);
        const includes = include.map(function (_) {
            return `**/${_}/**`;
        });

        if (!stats.isDirectory) throw new Error('The `src/node_modules` directory does not exist!');

        const paths = path.resolve(packagesDir, '{[^@]*/,@*/*/}');
        const packageDirs = yield glob$1(paths);
        const packageNames = packageDirs.filter(function (dir) {
            return includes.length > 0 ? includes.some(have(dir)) : true;
        }).filter(function (dir) {
            return !exclude.some(have(dir));
        }).map(function (dir) {
            return path.relative(path.resolve(root, 'src/node_modules'), dir);
        });

        return packageNames;
    });

    return function () {
        return _ref.apply(this, arguments);
    };
})();

var collapsePackages = ((all, curr) => {
    if (curr.undefined) {
        const und = all.undefined ? [...all.undefined, curr.undefined] : [curr.undefined];
        return _extends({}, all, {
            undefined: und
        });
    }
    return Object.assign(all, curr);
});

const cache$1 = {};

const getStructure = ({ root, name }) => {
    if (name in cache$1) return { [name]: cache$1[name] };

    const srcDir = path.relative(root, path.resolve(root, 'src/node_modules', name));
    const destDir = path.relative(root, path.resolve(root, 'dist', name));
    const files = readDir(path.resolve(root, srcDir));
    const structure = { name, root, srcDir, destDir, files };
    cache$1[structure.name] = structure;

    return { [name]: structure };
};

var getStructures = (({ root, packageNames = [] }) => packageNames.map(name => getStructure({ root, name })).reduce(collapsePackages, {}));

const type$3 = 'core-game';

const docs = 'Dropbox/My PC (Ray-PC)/Documents';
const coreDir = 'My Games/CORE/Saved/Maps';
const scriptsDir = 'Data/Scripts';

var config = (() => {
    var _ref = asyncToGenerator(function* ({
        manifest: { name: folder, dependencies },
        structure: { name, root, srcDir }
    }) {
        const destDir = `${docs}/${coreDir}/${folder}/${scriptsDir}`;

        const isNamespace = function isNamespace(dep) {
            return dep.startsWith('@') && dep.indexOf('/') < 0;
        };

        var _partition = partition__default['default'](dependencies, isNamespace),
            _partition2 = slicedToArray(_partition, 2);

        const include = _partition2[0],
              exact = _partition2[1];

        const namespaced = yield findPackages({ root, include });

        const deps = [...exact, ...namespaced];
        const depStructures = getStructures({ root, packageNames: deps });
        const depPosts = map__default['default'](depStructures, function (post) {
            return { type: 'lua-rock', post };
        });

        const externals = map__default['default'](depStructures, function ({ name: ex, destDir: exDest }) {
            return {
                src: `${exDest}/init.lua`,
                file: `${resolveName(ex)}.lua`,
                dest: destDir
            };
        });
        const post = { root, name, srcDir, destDir, externals };

        return [...depPosts, { type: type$3, post }];
    });

    return function (_x) {
        return _ref.apply(this, arguments);
    };
})();

var winRoot = (user => `/mnt/c/Users/${user}/`);

const cache$2 = {};

var manifest = (({ root, name, structure: { srcDir } }) => {
    if (name in cache$2) return cache$2[name];

    const corespecJson = path.resolve(root, srcDir, 'corespec.json');

    // eslint-disable-next-line global-require, import/no-dynamic-require
    const manifest = fileExists.sync(corespecJson) ? require(corespecJson) : {};
    cache$2[name] = manifest;

    return manifest;
});

const type$4 = 'core-game';

const corespecJson = (root, srcDir, files) => files.includes('corespec.json') ? // eslint-disable-next-line global-require, import/no-dynamic-require
require(path.resolve(root, srcDir, 'corespec.json')) : false;

const filter = ({ root = '.', srcDir = '.', files = [] }) => {
    const manifestData = corespecJson(root, srcDir, files);
    return !!manifestData;
};

var coreGame = {
    type: type$4,
    manifest,
    config,
    filter,
    build: () => {},
    postBuild: ({ config: { root, externals } }) => Promise.all(externals.map(({ src, dest, file }) => copy(path.resolve(root, src), path.resolve(winRoot('RayBenefield'), dest, file))))
};

var check = (() => {
    var _ref = asyncToGenerator(function* ({
        structure: { files = [] } = {},
        manifest: { version } = {}
    } = {}) {
        const checks = {
            'has a version number': !!version,
            'has index.md': files.includes('index.md')
        };

        return _extends({}, checks, { publishable: Object.values(checks).every(function (pass) {
                return pass;
            }) });
    });

    return function () {
        return _ref.apply(this, arguments);
    };
})();

var parseSfp = (({ content = '' } = {}) => {
    var _frontMatter = frontMatter__default['default'](content);

    const attributes = _frontMatter.attributes,
          body = _frontMatter.body;

    return {
        body: body.trim(),
        attributes,
        code: body.split('\n').reduce(({ inBlock, bundle }, line) => line.slice(0, 3) === '```' ? /* eslint-disable indent */
        { inBlock: !inBlock, bundle } : /* eslint-disable indent */
        inBlock ? {
            inBlock,
            bundle: bundle ? `${bundle}\n${line}` : line
        } : { inBlock, bundle }, { inBlock: false, bundle: '' }).bundle
    };
});

var entyre = (({
    readmePath = './readme.md',
    packageJsonPath = './package.json',
    config = {},
    readme = ''
} = {}) => ({
    name: 'entyre',
    transform(content, id) {
        if (path.extname(id) !== '.md') return { code: content };

        var _parse = parseSfp({ content });

        const body = _parse.body,
              code = _parse.code;
        var _parse$attributes = _parse.attributes;
        const attributes = _parse$attributes === undefined ? {} : _parse$attributes;

        readme = body;
        config = attributes;
        return { code };
    },
    onwrite() {
        return asyncToGenerator(function* () {
            return Promise.all([write$1(packageJsonPath, JSON.stringify(config, null, 4), 'utf-8'), write$1(readmePath, readme, 'utf-8')]);
        })();
    }
}));

const type$5 = 'entyre-package';
const rollupConfig$1 = { type: type$5, outFile: 'index.js', format: 'cjs' };

var entyrePackage = {
    type: type$5,
    filter: ({ root, name, srcDir = '.', files = [] }) => {
        if (files.includes('package.json')) {
            var _getJsDetails = getJsDetails({
                root,
                name,
                structure: { srcDir }
            }),
                _getJsDetails$main = _getJsDetails.main;

            const main = _getJsDetails$main === undefined ? false : _getJsDetails$main;

            return main && path.extname(main) === '.md';
        }
        return files.includes('index.md');
    },
    manifest: ({ root, structure: { srcDir } }) => read$1(path.resolve(root, srcDir, 'index.md'), 'utf-8').then(frontMatter__default['default']).then(({ attributes }) => attributes),
    config: snowball => [rollup(_extends({}, rollupConfig$1, {
        file: snowball.manifest.main || 'index.md',
        plugins: [entyre({
            readmePath: path.resolve(snowball.structure.destDir, 'readme.md'),
            packageJsonPath: path.resolve(snowball.structure.destDir, 'package.json')
        }), ...commonPlugins]
    }))],
    build,
    postBuild: () => {},
    check
};

const wait = ms => new Promise(resolve => setTimeout(resolve, ms));

var getContents = (({
    message = 'Create some contents: ',
    contents = '',
    pause = 2000
} = {}) => {
    log(`${_$2.green('?')} ${_$2.white.bold(`${message}: `)} ${_$2.grey(`Opening your editor in ${pause / 1000} secs...`)}`);
    return wait(pause).then(() => ({ contents: editor__default$1['default'].edit(contents) }));
});

const solvable = ['has readme file', 'has a version number'];
const fixers = {
    'has readme file': name => getContents({
        message: `Write your readme.md for ${name}`,
        contents: `# ${name}`
    }),
    'has a version number': () => Promise.resolve({ version: '0.0.0' })
};

var fix = transmuter().extend('fixable', ({ name, checks }) => Object.entries(checks.packages[name]).filter(([check]) => solvable.includes(check)).filter(([, passing]) => !passing).map(([fixable]) => fixable)).extend('fixes', ({ name, fixable }) => Promise.all(fixable.map(fix => fixers[fix](name).then(result => ({ [fix]: result })))).then(results => results.reduce(collapsePackages, {})));

var check$1 = (() => {
    var _ref = asyncToGenerator(function* ({
        structure: { files = [] } = {},
        manifest: { main, bin, version } = {}
    } = {}) {
        const hasIndex = files.includes('index.js');
        const checkBins = function checkBins() {
            return Object.values(bin).every(function (f) {
                return files.includes(f);
            });
        };
        const checks = _extends({
            'has readme file': files.includes('readme.md'),
            'has a version number': !!version
        }, hasIndex ? { 'has index': hasIndex } : {}, main ? { 'has main': files.includes(main) } : {}, bin ? { 'has bin': checkBins() } : {});

        return _extends({}, checks, { publishable: Object.values(checks).every(function (pass) {
                return pass;
            }) });
    });

    return function () {
        return _ref.apply(this, arguments);
    };
})();

const preCreate = ({ packagePath, name }) => inquirer.prompt([{
    type: 'input',
    name: 'executableName',
    message: 'What should the executable name be?',
    default: name
}, {
    type: 'editor',
    name: 'executableContents',
    message: 'What should the initial contents be for the executable?',
    default: dedent__default['default'](`
                import log from '@dev-xp/log'

                log('${name}')
            `)
}]).then(({ executableName, executableContents }) => ({
    cliPath: path.join(packagePath, 'cli.js'),
    manifestPath: path.join(packagePath, 'package.json'),
    manifestContents: { bin: { [executableName]: 'cli.js' } },
    executableName,
    executableContents,
    scriptPath: `scripts.${executableName}`,
    script: `cd src/node_modules && babel-node ${name}/cli.js`
}));

var create = (({
    rootManifestPath,
    packagePath,
    createFiles: {
        cliPath,
        manifestPath,
        manifestContents,
        executableContents,
        scriptPath,
        script
    } = {}
} = {}) => {
    mkdir(packagePath);
    writeSync(manifestPath, manifestContents);
    write$1(cliPath, executableContents);
    edit(rootManifestPath, scriptPath, script);
});

const banner = '#!/usr/bin/env node';
const type$6 = 'npm-cli-tool';
const plugins$1 = [...commonPlugins, exec__default['default']()];
const rollupConfig$2 = { type: type$6, plugins: plugins$1, banner, format: 'cjs' };

var npmCli = {
    type: type$6,
    details: () => ({ main: 'cli.js' }),
    filter: ({ root = '.', name = '', srcDir = '.', files = [] }) => {
        if (files.includes('package.json')) {
            var _getJsDetails = getJsDetails({
                root,
                name,
                structure: { srcDir }
            }),
                _getJsDetails$bin = _getJsDetails.bin;

            const bin = _getJsDetails$bin === undefined ? false : _getJsDetails$bin;

            return bin && bin.constructor === Object;
        }
        return false;
    },
    manifest: getJsDetails,
    config: snowball => Object.values(snowball.manifest.bin).map(file => rollup(_extends({}, rollupConfig$2, snowball, { file }))),
    build,
    postBuild,
    check: check$1,
    fix,
    preCreate,
    create,
    createTemplate: ({
        createFiles: { cliPath, executableName, script }
    } = {}) => `
        <attr><key>CLI Path:</key><value>${cliPath}</value></attr>
        <attr><key>Executable Name:</key><value>${executableName}</value></attr>
        <attr><key>Run Script:</key><value>${script}</value></attr>
    `
};

const NOOP = () => {};
const NOFN = () => () => {};
const NOOBJ = () => ({});
const NOSTR = () => ``;

var getTypes = (({ plugAll }) => ({ structures = {} }) => {
    const getFirstTrue = _.pipe(plugAll('filter'), _.filter(_$1.isTrue), _.keys, _.head);
    return _.map(getFirstTrue, structures);
});

var extraDetails = (({ plug = NOFN } = {}) => (name = '<undefined>', { types = {}, structures = {}, manifests = {} } = {}) => ({
    [name]: plug('details')({
        type: types[name],
        structure: structures[name],
        manifest: manifests[name]
    })
}));

// eslint-disable-next-line global-require, import/no-dynamic-require
var getRootManifest = (({ root = '.' } = {}) => require(path.resolve(root, 'package.json')));

var getPublishing = (({ destination = {} } = {}) => Object.entries(destination).filter(([, { destVersion, srcVersion }]) => semver.valid(srcVersion) ? semver.valid(destVersion) ? semver.gt(srcVersion, destVersion) : true : false).map(([name]) => name));

const requiredProps$1 = ['description', 'homepage', 'author', 'repository', 'bugs', 'license', 'dependencies'];
const isIn = obj => partial__default['default'](has__default['default'], obj);

var publishCheck = (({ plugins = {} } = {}) => ({
    rootStructure: { files = [] } = {},
    rootManifest = {},
    packageNames = [],
    types = {},
    structures = {},
    manifests = {}
} = {}) => Promise.all(packageNames.map(name => {
    const noChecks = { [name]: {} };
    if (!plugins[types[name]]) return noChecks;
    if (!plugins[types[name]].check) return noChecks;

    return plugins[types[name]].check({
        structure: structures[name],
        manifest: manifests[name]
    }).then(checks => ({ [name]: checks }));
})).then(checks => checks.reduce(collapsePackages, {})).then(packageChecks => ({
    root: {
        'has root license': files.includes('LICENSE'),
        'has required props': every__default['default'](requiredProps$1, isIn(rootManifest))
    },
    packages: packageChecks
})));

const regex = /#(\d*)/;

var parseTags = (({ rawTags = [] } = {}) => rawTags.map(tag => tag.match(regex) || []).filter(matches => matches.length).map(([tag, releaseNumber]) => ({
    tag,
    releaseNumber: parseInt(releaseNumber, 10)
})));

const git = configureGit__default['default']();

var gatherTags = (() => git.tags().then(({ all }) => all));

const formatPackage = ({ type, scopes, subject }) => `
    <change>
        <type>${type}</type>
        <scope>(${scopes.join(',')})<scope>: 
        <subject>${subject}</subject>
    </change>
`;

var template = (() => ({ changes }) => `
    <bar>-</bar>
    <h1>Changes</h1>
    <bar>-</bar>
    <list>${changes.sort(() => 1).map(formatPackage).join('')}</list>
`);

/* eslint-disable indent */
const regex$1 = /^([^A-Z]*)\(((?:[^A-Z*]*)|\*)\): ([^a-z].*\.)$/;

const toHeaderOrBody = ({ body, matches: [, type, scopes, subject] }) => type ? {
    type,
    scopes: scopes.split(',').map(s => s.trim()),
    subject
} : { body };

const intoSingleCommits = (combined, line) => {
    if (combined.length === 0) return [line];

    var _combined$reverse = combined.reverse(),
        _combined$reverse2 = toArray(_combined$reverse);

    const last = _combined$reverse2[0],
          rest = _combined$reverse2.slice(1);

    // We use `in` since body can be an empty string,
    // which returns falsy in (last.body && line.body)


    if ('body' in last && 'body' in line) {
        return [...rest, _extends({}, last, {
            body: [last.body, line.body].join('\n')
        })];
    }

    if (line.type) {
        return [...combined, line];
    }

    return [...rest, _extends({}, last, line)];
};

const trimmedBodies = combined => combined.body ? _extends({}, combined, { body: combined.body.trim() }) : combined;

var parse = (({ commits = [] } = {}) => Array.prototype.concat(...commits.map(({ hash, message, files }) => message.split(/\r?\n/).map(line => ({
    body: line,
    matches: line.match(regex$1) || []
})).map(toHeaderOrBody).reduce(intoSingleCommits, []).map(trimmedBodies).map(commit => _extends({ hash, files }, commit)))));

const git$1 = configureGit__default['default']();

// https://git-scm.com/docs/pretty-formats
var gatherChanges = (({ from, to = 'HEAD' } = {}) => git$1.log({
    from,
    to,
    format: {
        hash: '%H',
        message: '%B'
    }
}).then(({ all }) => Promise.all(all.map(commit => git$1.show(['--pretty=', '--name-only', commit.hash]).then(files => files.trim().split('\n')).then(files => _extends({}, commit, { files }))))).then(commits => ({ commits })).then(parse));

var latestReleaseTag = (({ tags = [] } = {}) => tags.filter(tag => tag.releaseNumber).reduce((latest, current) => latest.releaseNumber > current.releaseNumber ? latest : current, { tag: 'HEAD', releaseNumber: 0 }));

function changes() {
    return transmuter({}).extend('rawTags', gatherTags).extend('tags', parseTags).extend('latest', latestReleaseTag).extend('from', ({ latest }) => latest.tag).extend('changes', gatherChanges);
}

const command = () => ({
    name: 'changes',
    aliases: ['changelog'],
    run: changes,
    template
});

const cache$3 = {};

var detective = (({ root, locals = [], structures = {}, manifests = {} }) =>
// eslint-disable-next-line complexity
(() => {
    var _ref = asyncToGenerator(function* (name) {
        if (!name) return {};

        var _ref2 = manifests[name] || {};

        const main = _ref2.main,
              bin = _ref2.bin;

        var _ref3 = structures[name] || {};

        const srcDir = _ref3.srcDir,
              files = _ref3.files;


        const sourceFile = main || (bin ? Object.values(bin)[0] : files.includes('index.md') ? 'index.md' : 'index.js');

        if (!fileExists.sync(path.resolve(root, srcDir, sourceFile))) return {};

        return read$1(path.resolve(root, srcDir, sourceFile), 'utf-8').then(function (content) {
            return path.extname(sourceFile) === '.md' ? parseSfp({ content }).code : content;
        }).then(detective__default['default']).then(function (result) {
            return Promise.all(result.map((() => {
                var _ref4 = asyncToGenerator(function* (dependency) {
                    if (!locals.includes(dependency)) return { [dependency]: false };

                    if (dependency in cache$3) return cache$3[dependency];

                    return detect(dependency).then(function (deps) {
                        return Object.keys(deps).length === 0 && deps.constructor === Object ? { [dependency]: false } : { [dependency]: deps };
                    }).then(function (tree) {
                        cache$3[dependency] = tree;
                        return tree;
                    });
                });

                return function (_x2) {
                    return _ref4.apply(this, arguments);
                };
            })())).then(function (results) {
                return results.reduce(collapsePackages, {});
            });
        });
    });

    function detect(_x) {
        return _ref.apply(this, arguments);
    }

    return detect;
})());

var getPackageDetails = (({ plug }) => ({
    root = '.',
    packageNames = [],
    types = {},
    structures = {}
}) => Promise.all(_.map(name => {
    const promise = plug('manifest')({
        type: types[name],
        root,
        name,
        structure: structures[name]
    });
    return Promise.resolve(promise).then(manifest => ({
        [name]: manifest
    }));
}, packageNames)).then(_.mergeAll));

var getDependencies = (config => (() => {
    var _ref = asyncToGenerator(function* ({ root, exclude, packageNames = [] } = {}) {
        if (!root) return {};
        if (packageNames.length <= 0) return {};

        const packageBasics = transmuter({ root, exclude }).extend('packageNames', findPackages).extend('structures', getStructures).extend('types', getTypes(config)).extend('manifests', getPackageDetails(config));

        var _ref2 = yield packageBasics;

        const locals = _ref2.packageNames,
              structures = _ref2.structures,
              manifests = _ref2.manifests;


        const findDependencies = detective({ root, locals, structures, manifests });
        return Promise.all(packageNames.map(function (name) {
            return findDependencies(name).then(function (deps) {
                return Object.entries(deps).reduce(function addDeps(list, [dep, depsOfDep]) {
                    return [...list, dep, ...Object.entries(depsOfDep).reduce(addDeps, [])];
                }, []);
            }).then(uniq__default['default']).then(function (deps) {
                return {
                    [name]: {
                        local: deps.filter(function (dep) {
                            return locals.includes(dep);
                        }),
                        external: deps.filter(function (dep) {
                            return !locals.includes(dep);
                        })
                    }
                };
            });
        })).then(function (results) {
            return results.reduce(collapsePackages, {});
        });
    });

    return function () {
        return _ref.apply(this, arguments);
    };
})());

const fields = ['version'];

var destinationDetails = (({ root, packageNames, structures }) => Promise.all(packageNames.map(name => Promise.all([
// TODO: Replace with API call
execa__default['default']('npm', ['view', name, ...fields]).then(result => result.stdout).catch(() => null), new Promise(res => {
    const packageJson = path.resolve(root, structures[name].srcDir, 'package.json');
    // eslint-disable-next-line global-require, import/no-dynamic-require
    return res(require(packageJson).version);
}).catch(() => null)]).then(([destVersion, srcVersion]) => ({
    [name]: { destVersion, srcVersion }
})))).then(results => results.reduce(collapsePackages, {})));

// eslint-disable-next-line complexity
var filterPackages = (config => snowball => {
    const packageNames = config.packageNames,
          allPackages = config.allPackages,
          onlyPublishable = config.onlyPublishable,
          onlyChanged = config.onlyChanged,
          changedAndPublishable = config.changedAndPublishable,
          needsPublishing = config.needsPublishing;


    if (allPackages) return findPackages(snowball);

    if (Array.isArray(packageNames) && packageNames.length > 0) return packageNames;

    if (!needsPublishing && !changedAndPublishable && !onlyPublishable && !onlyChanged) return findPackages(snowball);

    const packageBasics = transmuter(snowball).extend('packageNames', findPackages).extend('structures', getStructures).extend('types', getTypes(config)).extend('manifests', getPackageDetails(config));

    if (needsPublishing) return packageBasics.extend('checks', publishCheck(config)).extend('publishable', ({ packageNames: names, checks }) => names.filter(name => checks.packages[name].publishable)).extend('destination', destinationSnowball => destinationDetails(_extends({}, destinationSnowball, {
        packages: destinationSnowball.publishable
    }))).extend('publishing', getPublishing).then(({ publishing }) => publishing);

    if (onlyPublishable) return packageBasics.extend('checks', publishCheck(config)).then(({ packageNames: names, checks }) => names.filter(name => checks.packages[name].publishable));

    const packageChanges = packageBasics.extend(() => changes()).extend('dependencies', getDependencies(config)).extend('changed', ({ packageNames: names, dependencies, changes }) => names.filter(name => changes.some(({ scopes = [] }) => scopes.includes(name) ||
    // eslint-disable-next-line max-nested-callbacks
    scopes.some(scope => dependencies[name].local.includes(scope)))));

    if (changedAndPublishable) return packageChanges.extend('checks', publishCheck(config)).then(({ changed: names, checks }) => names.filter(pkg => checks.packages[pkg].publishable));

    if (onlyChanged) return packageChanges.then(({ changed }) => changed);

    return findPackages(snowball);
});

const formatPackage$1 = manifests => pkg => {
    var _ref = manifests[pkg] || {},
        _ref$version = _ref.version;

    const version = _ref$version === undefined ? 'No Version' : _ref$version;


    return `<item>
        <attr>
            <key>Name:</key>
            <value>${pkg}</value>
        </attr>
        <attr>
            <key>Version:</key>
            <value>${version}</value>
        </attr>
    </item>`;
};

var template$1 = (() => ({ packageNames = [], manifests = {} }) => `
    <bar>-</bar>
    <h1>Package List</h1>
    <bar>-</bar>
    <list>${packageNames.map(formatPackage$1(manifests)).join('')}</list>
`);

var getRootStructure = (({ root }) => ({ files: readDir(root) }));

function list(config) {
    return transmuter({}).extend('root', findRepoRoot).extend('rootStructure', getRootStructure).extend('rootManifest', getRootManifest).extend('exclude', ['**/fixtures/**']).extend('packageNames', filterPackages(config)).extend('structures', getStructures).extend('types', getTypes(config)).extend('manifests', getPackageDetails(config)).extendEach('details', 'packageNames', extraDetails(config));
}

const packageFilters = {
    _: 'packageNames',
    all: 'allPackages',
    publishable: 'onlyPublishable',
    changed: 'onlyChanged',
    releasing: 'changedAndPublishable',
    publishing: 'needsPublishing'
};

const command$1 = () => ({
    name: 'list',
    aliases: ['ls'],
    run: list,
    template: template$1,
    args: packageFilters
});

const NONE = '--- None ---';
const formatPackage$2 = dependencies => name => `<item>
    <title>${name}</title>
    <attr>
        <key>Local:</key>
        <value>${dependencies[name].local.join(', ') || NONE}</value>
    </attr>
    <attr>
        <key>External:</key>
        <value>${dependencies[name].external.join(', ') || NONE}</value>
    </attr>
</item>`;

var template$2 = (() => ({ packageNames, dependencies }) => `
    <bar>-</bar>
    <h1>Package Dependencies</h1>
    <bar>-</bar>
    <list>${packageNames.map(formatPackage$2(dependencies)).join('')}</list>
`);

function dependencies(config) {
    return list(config).extend('dependencies', getDependencies(config));
}

const command$2 = () => ({
    name: 'dependencies',
    aliases: ['deps', 'dependents'],
    run: dependencies,
    template: template$2,
    args: packageFilters
});

var patchObject = (patches => _.mapObjIndexed((old, key) => {
    if (!_.has(key, patches)) return old;

    return (...args) => {
        const patch = _.view(_.lensProp(key), patches);
        const result = old(...args);

        return result.then ? result.then(patch) : patch(result);
    };
}));

const camelCaseKeys = obj => {
    if (lodash.isArray(obj)) return obj.map(camelCaseKeys);

    if (lodash.isObject(obj)) {
        const newObj = lodash.mapValues(obj, camelCaseKeys);
        return lodash.mapKeys(newObj, (_, key) => lodash.camelCase(key));
    }

    return obj;
};

const patch = patchObject({ status: camelCaseKeys });
const makeGit = () => patch(configureGit__default['default'](sync()));

makeGit();

var git$2 = makeGit().outputHandler((command, stdout, stderr) => {
    stdout.pipe(process.stdout);
    stderr.pipe(process.stderr);
});

const check$2 = (bool, msg) => bool ? `<success>${msg}</success>` : `<fail>${msg}</fail>`;

const dryMsg = `
    <bar>-</bar>
    <h1><warning>DRY RUN... nothing created</warning></h1>
    <bar>-</bar>
`;

var template$3 = (({ dry }) => ({
    monorepoName,
    root,
    isGitDir,
    packageJsonExists,
    modulesDirExists
}) => `
    <bar>-</bar>
    <h1>[${monorepoName}] Monorepo Initialization</h1>
    <bar>-</bar>
    <h1>Directory Root: <details>${root}</details></h1>
    <list>
        <bullet>
            ${check$2(isGitDir, 'Is a Git Directory')}
        </bullet>
        <bullet>
            ${check$2(packageJsonExists, 'Has a package.json')}
        </bullet>
        <bullet>
            ${check$2(modulesDirExists, 'Has a src/node_modules directory')}
        </bullet>
    </list>
    ${dry ? dryMsg : ''}
`);

const isGitDirCheck = dir => exists(path.join(dir, '.git'));
const packageJsonExistsCheck = dir => exists(path.join(dir, 'package.json'));
const modulesDirExistsCheck = dir => exists(path.join(dir, 'src/node_modules'));

function collectInfo() {
    return transmuter({}).extend('root', findRepoRoot()).extend('monorepoName', ({ root }) => path.basename(root)).extend('isGitDir', ({ root }) => isGitDirCheck(root)).extend('packageJsonExists', ({ root }) => packageJsonExistsCheck(root)).extend('modulesDirExists', ({ root }) => modulesDirExistsCheck(root));
}

function initialize() {
    // eslint-disable-next-line complexity
    return ({
        root,
        monorepoName,
        isGitDir,
        packageJsonExists,
        modulesDirExists
    }) => {
        if (isGitDir && packageJsonExists && modulesDirExists) throw new Error(`[${monorepoName}] is already a KIKD monorepo.`);

        if (!isGitDir) git$2.init();

        const manifestPath = path.join(root, 'package.json');
        if (!packageJsonExists) {
            write(manifestPath, { name: monorepoName });
            log('Package.json created.');
        }

        const modulesPath = path.join(root, 'src/node_modules');
        if (!modulesDirExists) {
            mkdir(modulesPath);
            log('src/node_modules created.');
        }
    };
}

const command$3 = () => ({
    name: 'initialize',
    run: collectInfo,
    effect: initialize,
    template: template$3,
    autoCommit: true
});

var template$4 = (() => ({ type, packagePath, createTemplate }) => `
    <bar>-</bar>
    <h1>New Package</h1>
    <bar>-</bar>
    <attr><key>Type:</key><value>${type}</value></attr>
    <attr><key>Path:</key><value>${packagePath}</value></attr>
    ${createTemplate}
`);

var requestPackageType = (({ plugins = {} } = {}) => () => inquirer.prompt([{
    type: 'list',
    name: 'type',
    message: 'What kind of package would you like to create?',
    choices: Object.values(plugins).filter(plugin => 'create' in plugin).map(({ type }) => type)
}]).then(({ type }) => type));

var requestPackageName = (({ packageName }) => () => inquirer.prompt([{
    type: 'input',
    name: 'name',
    default: packageName[0],
    message: 'What is the name of this new package?'
}]).then(({ name }) => name));

function collectInfo$1(config = {}) {
    var _config$plug = config.plug;
    const plug = _config$plug === undefined ? NOFN : _config$plug;

    return transmuter({}).extend('root', findRepoRoot).extend('rootManifestPath', ({ root }) => path.join(root, 'package.json')).extend('type', requestPackageType(config)).extend('name', requestPackageName(config)).extend('packagePath', ({ root, name }) => path.join(root, 'src/node_modules', name)).extend('createFiles', plug('preCreate')).extend('createTemplate', plug('createTemplate'));
}

const command$4 = () => ({
    name: 'new',
    run: collectInfo$1,
    template: template$4,
    autoCommit: true,
    plugs: { preCreate: NOOBJ, create: NOOP, createTemplate: NOSTR },
    args: { _: 'packageName' },
    effect: ({ plug = NOFN } = {}) => plug('create')
});

const buildResult = ({ src, file }) => `<finer>${src} -> ${file}</finer>`;

const formatPackage$3 = ({
    post: { name, srcDir, destDir, externals = [] } = {},
    type
} = {}) => `
        <item>
            <title>${name} (${type})</title>
            <details>${srcDir} -> ${destDir}</details>
            ${externals.length > 0 ? '<finer>-</finer>' : ''}
            ${externals.map(buildResult).join('')}
            ${externals.length > 0 ? '<finer>-</finer>' : ''}
        </item>
   `;

var template$5 = (() => ({ configs }) => `
    <bar>-</bar>
    <h1>Packages to be Built</h1>
    <bar>-</bar>
    <list>
        ${[].concat(...Object.values(configs)).map(formatPackage$3).join('')}
    </list>
`);

var buildPackages = (({ plug = NOFN } = {}) => ({
    root,
    rootManifest,
    configs = {}
} = {}) => {
    const allConfigs = [].concat(...Object.values(configs));

    const buildPackages = () => Promise.all(allConfigs.map(config => ({ root, config, type: config.type })).map(plug('build')));
    const postBuildPackages = () => Promise.all(allConfigs.map(config => ({
        root,
        rootManifest,
        config: config.post,
        type: config.type
    })).map(plug('postBuild')));

    return buildPackages().then(postBuildPackages);
});

var buildConfigs = (({ plug = NOFN } = {}) => (name = '<undefined>', {
    types = {},
    rootManifest = {},
    dependencies = {},
    structures = {},
    manifests = {}
} = {}) => ({
    [name]: plug('config')({
        type: types[name],
        rootManifest,
        dependencies: dependencies[name],
        structure: structures[name],
        manifest: manifests[name]
    })
}));

const filter$1 = ({ allPackages, changedAndPublishable, needsPublishing }) => !allPackages && !changedAndPublishable && !needsPublishing ? { onlyPublishable: true } : {};

function build$1(config) {
    const newConfig = _extends({}, config, filter$1(config));

    return list(newConfig).extend('dependencies', dependencies(newConfig).then(isolate('dependencies'))).extendEach('configs', 'packageNames', buildConfigs(newConfig));
}

const command$5 = () => ({
    name: 'build',
    run: build$1,
    template: template$5,
    effect: buildPackages,
    args: packageFilters
});

const testChecks = pkgChecks => Object.entries(pkgChecks).filter(([check]) => check !== 'publishable').map(([check, result]) => result ? `<check><success>${check}</success></check>` : `<check><fail>${check}</fail></check>`).join('');

const publishableCheck = publishable => publishable ? '<success>publishable</success>' : '<fail>not publishable</fail>';

const formatPackage$4 = ([name, pkgChecks]) => `<item>
    <title>${name}</title>
    ${testChecks(pkgChecks)}
    <check>${Array(23).join('─')}</check>
    <check>${publishableCheck(pkgChecks.publishable)}</check>
</item>`;

const rootChecks = checks => Object.entries(checks.root).map(([check, result]) => result ? `<check><success>${check}</success></check>` : `<check><fail>${check}</fail></check>`).join('');

var template$6 = (() => ({ checks }) => `
    <bar>-</bar>
    <h1>Checked Packages</h1>
    <bar>-</bar>
    <list>
        ${Object.entries(checks.packages).map(formatPackage$4).join('')}
    </list>
    <bar>-</bar>
    <title>Repository</title>
    ${rootChecks(checks)}
`);

function check$3(config) {
    return list(config).extend('checks', publishCheck(config));
}

const command$6 = () => ({
    name: 'check',
    run: check$3,
    template: template$6,
    args: packageFilters
});

const preCreate$1 = ({ name, packagePath }) => inquirer.prompt([{
    type: 'editor',
    name: 'indexContents',
    message: 'What should the initial contents be for the index be?',
    default: dedent__default['default']`
                import log from '@dev-xp/log'

                export default () => log('${name}')
            `
}, {
    type: 'confirm',
    name: 'makeTest',
    message: 'Would you like a `kape` snapshot test made?',
    default: true
}]).then(({ indexContents, makeTest = true }) => _extends({
    indexPath: path.join(packagePath, 'index.js'),
    indexContents
}, makeTest && {
    testPath: path.join(packagePath, 'snapshot-tests.js'),
    testContents: dedent__default['default']`
                import describe from 'kape'
                import ${lodash.camelCase(resolveName(name))} from '${name}'

                describe('${name}', ${lodash.camelCase(resolveName(name))}, snapshot => {
                    snapshot([[]])
                })
            `
}));

const createTemplate = ({ createFiles: { indexPath } }) => `
    <attr><key>Index Path:</key><value>${indexPath}</value></attr>
`;

var create$1 = (({
    packagePath,
    createFiles: { indexPath, indexContents, testPath, testContents } = {}
} = {}) => {
    mkdir(packagePath);
    write$1(indexPath, indexContents);
    if (testPath) write$1(testPath, testContents);
});

const type$7 = 'npm-package';
const rollupConfig$3 = { type: type$7, plugins: commonPlugins, format: 'cjs' };

var npmPackage = {
    type: type$7,
    details: () => ({ main: 'index.js' }),
    filter: ({ root = '.', srcDir = '.', files = [] }) => {
        if (files.includes('package.json')) {
            // eslint-disable-next-line global-require, import/no-dynamic-require
            const manifest = require(path.resolve(root, srcDir, 'package.json'));
            if (!manifest.main) return files.includes('index.js');
            return path.extname(manifest.main) === '.js' || files.includes('index.js');
        }
        return files.includes('index.js');
    },
    manifest: getJsDetails,
    config: snowball => {
        var _ref = snowball.manifest || {},
            _ref$file = _ref.file;

        const file = _ref$file === undefined ? 'index.js' : _ref$file;

        return rollup(_extends({}, rollupConfig$3, snowball, { file, exports: 'named' }));
    },
    build,
    postBuild,
    check: check$1,
    fix,
    preCreate: preCreate$1,
    create: create$1,
    createTemplate
};

const registry = registryUrl__default['default']();

const setAuthToken = root => new Promise((res, rej) => {
    process.chdir(root);
    const npmrcFile = localOrHomeNpmrc__default['default']();
    if (!exists(npmrcFile)) return rej(new Error('For some reason no npmrc file exists.'));

    const test = `${registry.replace('https:', '').replace('http:', '')}:_authToken=`;

    // eslint-disable-next-line prefer-template, no-template-curly-in-string
    const token = test + '${NPM_TOKEN}';
    // eslint-disable-next-line prefer-template
    const contents = readSync(npmrcFile, 'utf-8').trim() + '\n';

    if (contents.indexOf(token) !== -1) {
        return res();
    }

    if (contents.indexOf(test) !== -1) {
        err('The .npmrc already has a token for this registry.');
        err(test);
        return rej(new Error(`Auth token for registry exists ${test}`));
    }

    writeSync$1(npmrcFile, `${contents}${token}\n`, 'utf-8');
    return res();
});

var deployNpm = (({ root, publishing }) => setAuthToken(root).then(() => Promise.all(publishing.map(name => {
    process.chdir(path.resolve(root, 'dist', name));
    log(`Publishing [${name}]...`);
    return execa__default['default']('npm', ['publish', '--registry', registry, '--access', 'public']).then(result => log(result.stdout)).catch(log);
}))));

const publishableCheck$1 = (publishing, name) => publishing.includes(name) ? `<success>${name}</success>` : `<fail>${name}</fail>`;

const formatPackage$5 = (destination, publishing) => name => `<item>
    <check>
        ${publishableCheck$1(publishing, name)}
        <attr>
            <key>NPM:</key>
            <value>${destination[name].destVersion}</value>
        </attr>
        <attr>
            <key>Next:</key>
            <value>${destination[name].srcVersion}</value>
        </attr>
    </check>
</item>`;

var template$7 = (() => ({ packageNames, destination, publishing }) => `
    <bar>-</bar>
    <h1>Publishable Packages</h1>
    <bar>-</bar>
    <list>
        ${packageNames.map(formatPackage$5(destination, publishing)).join('')}
    </list>
`);

function publish(config) {
    return list(_extends({}, config, { onlyPublishable: true })).extend('destination', destinationDetails).extend('publishing', getPublishing);
}

const command$7 = () => ({
    name: 'publish',
    aliases: ['deploy'],
    run: publish,
    template: template$7,
    // eslint-disable-next-line complexity
    effect: () => config => {
        var _process$env = process.env;
        const NPM_TOKEN = _process$env.NPM_TOKEN,
              GH_USER = _process$env.GH_USER,
              GH_EMAIL = _process$env.GH_EMAIL,
              GH_REPO = _process$env.GH_REPO,
              GH_TOKEN = _process$env.GH_TOKEN;


        if (!NPM_TOKEN) throw new Error('The NPM_TOKEN env variable must be set!');

        if (!GH_USER) throw new Error('The GH_USER env variable must be set!');
        if (!GH_EMAIL) throw new Error('The GH_EMAIL env variable must be set!');
        if (!GH_REPO) throw new Error('The GH_REPO env variable must be set!');
        if (!GH_TOKEN) throw new Error('The GH_TOKEN env variable must be set!');

        return deployNpm(config);
    }
});

var setupGit = (() => {
    var _process$env = process.env;
    const GH_USER = _process$env.GH_USER,
          GH_EMAIL = _process$env.GH_EMAIL,
          GH_REPO = _process$env.GH_REPO,
          GH_TOKEN = _process$env.GH_TOKEN;


    log('Setting committer...');
    return git$2.addConfig('user.name', GH_USER).then(() => git$2.addConfig('user.email', GH_EMAIL)).then(() => log('Setting up branch...')).then(() => git$2.addRemote('origin-deploy', `https://${GH_TOKEN}@github.com/${GH_REPO}.git`)).then(() => git$2.fetch('origin-deploy', 'master')).then(() => git$2.checkout(['-B', 'deploy-master', '--track', 'origin-deploy/master'])).then(() => git$2.reset(['--hard', 'origin-deploy/master']));
});

var deployGit = (() => {
    log('Pushing to deploy remote...');
    return git$2.push(['origin-deploy', 'HEAD:master', '--tags']);
});

const git$3 = configureGit__default['default']();

var updatePackages = (({
    root,
    updates,
    packageNames = [],
    structures = {},
    latest
}) => {
    const newReleaseNumber = latest.releaseNumber + 1;
    log(`Starting release #${newReleaseNumber}`);

    log('Updating package.json files...');
    return Promise.all(packageNames.filter(name => name in updates).map(name => {
        var _ref = structures[name] || {};

        const srcDir = _ref.srcDir;

        const jsonPath = path.resolve(root, srcDir, 'package.json');
        return read(jsonPath).then(config => _extends({}, config, { version: updates[name].new })).then(config => write(jsonPath, config)).then(() => ({
            name,
            version: updates[name].new,
            file: jsonPath
        }));
    })).then((() => {
        var _ref2 = asyncToGenerator(function* (packageUpdates) {
            log('Staging files for commit...');
            yield git$3.add(map__default['default'](packageUpdates, 'file'));
            return packageUpdates;
        });

        return function (_x) {
            return _ref2.apply(this, arguments);
        };
    })()).then((() => {
        var _ref3 = asyncToGenerator(function* (packageUpdates) {
            log('Committing updates...');
            yield git$3.commit(`release(${newReleaseNumber}): Update package versions for ${packageUpdates.length > 1 ? `${packageUpdates.length} packages.` : '1 package.'}
        
${packageUpdates.map(function ({ name, version }) {
                return `v${version} ${name}`;
            }).join('\n')}

[skip ci]
`);
            return packageUpdates;
        });

        return function (_x2) {
            return _ref3.apply(this, arguments);
        };
    })()).then(packageUpdates => {
        log('Tagging commit with package versions...');
        return Promise.all(packageUpdates.map(({ name, version }) => git$3.addTag(`${name}-v${version}`)));
    }).then(() => {
        log('Tagging commit with new latest release tag...');
        return git$3.addTag(`#${newReleaseNumber}`);
    });
});

const formatPackage$6 = ([name, { old, new: version, type, reason }]) => `
    <item>
        <title>${name} - ${type}</title>
        <check>${reason}</check>
        <check>${old} -> ${version}</check>
    </item>
`;

var template$8 = (() => ({ updates }) => `
    <bar>-</bar>
    <h1>Packages to be updated</h1>
    <bar>-</bar>
    <list> ${Object.entries(updates).map(formatPackage$6).join('')}</list>
`);

const createPatch = ({ name, version, reason }) => ({
    [name]: {
        type: 'patch',
        old: version,
        new: semver.inc(version, 'patch'),
        reason
    }
});

var recommendVersions = ((
// eslint-disable-next-line no-unused-vars
{ packageNames = [], dependencies = {}, manifests = {}, changes = [] } = {}) => {
    if (packageNames.length <= 0) return {};

    return packageNames
    // eslint-disable-next-line complexity
    .map(name => {
        var _ref = manifests[name] || {};

        const version = _ref.version;

        /*
         Packages without a `version` field in their `package.json` will not
        be considered for versioning.
         */

        if (!version) return undefined;

        /*
         Packages with a `version` set to `0.0.0` will increase their `patch`
        version as their initial public release and become `0.0.1` no matter
        what other changes exist.
         */
        if (version === '0.0.0') {
            return {
                [name]: {
                    type: 'patch',
                    old: version,
                    new: '0.0.1',
                    reason: 'Prereleased'
                }
            };
        }

        /*
         Packages with a `version` less than `0.1.0` will increase their
        `patch` version if they or any of their dependencies are updated due
        to being in "development".
         */
        if (semver.lt(version, '0.1.0')) {
            const pkgChanges = filter__default['default'](changes, change => change.scopes ? change.scopes.includes(name) : false);

            if (pkgChanges.length > 0) return createPatch({
                name,
                version,
                reason: `This had ${pkgChanges.length === 1 ? '1 change' : `${pkgChanges.length} changes`} while in development.`
            });

            if (!dependencies[name]) return undefined;

            const depChanges = filter__default['default'](changes, change => change.scopes ? intersection__default['default'](dependencies[name].local, change.scopes).length > 0 : false);

            if (depChanges.length > 0) return createPatch({
                name,
                version,
                reason: `There ${depChanges.length === 1 ? 'was 1 change' : `were ${depChanges.length} changes`} in dependencies while in development.`
            });

            return undefined;
        }

        return undefined;
    }).filter(patch => patch).reduce(collapsePackages, {});
});

const CI = process.env.CI;


function version(config) {
    return list(_extends({}, config, { changedAndPublishable: true })).extend(changes().then(isolate(['changes', 'latest']))).extend('dependencies', dependencies(config).then(isolate('dependencies'))).extend('updates', recommendVersions);
}

const command$8 = () => ({
    name: 'version',
    aliases: ['update'],
    run: version,
    template: template$8,
    effect: () => config => {
        if (!CI) throw new Error('Committing new versions should only happen in CI.');

        return setupGit().then(() => updatePackages(config)).then(() => deployGit());
    }
});

const fixers$1 = {
    'has readme file': ({ root, structure: { srcDir }, contents }) => {
        log('Creating [readme.md]...');
        return write$1(path.resolve(root, srcDir, 'readme.md'), contents);
    },
    'has a version number': ({ root, structure: { srcDir }, version }) => {
        log('Creating [package.json]...');
        return write(path.resolve(root, srcDir, 'package.json'), { version });
    }
};

var effect = (() => ({ root, structure, fixes }) => Promise.all(Object.entries(fixes).map(([fix, details]) => fixers$1[fix](_extends({ root, structure }, details)))));

const templates = {
    'has readme file': 'A readme will be created.',
    'has a version number': 'A version will be added.'
};

var template$9 = (() => ({ name, fixes }) => `
    <bar>-</bar>
    <h1>Prereleasing [${name}]</h1>
    <bar>-</bar>
    <list>
    ${Object.entries(fixes).map(([type]) => `<check><success>${templates[type]}</success></check>`).join('')}
    </list>
`);

inquirer__default['default'].registerPrompt('autocomplete', autocomplete__default['default']);

const searchPackages = packageNames => (answers, input = '') => new Promise(res => {
    setTimeout(() => res(fuzzy.filter(input, packageNames).map(result => result.original || result)), 500);
});

var pickPackage = (({ message = 'Which package should we pick?', packageNames }) => inquirer__default['default'].prompt([{
    type: 'autocomplete',
    name: 'name',
    message,
    source: searchPackages(packageNames)
}]).then(({ name }) => name));

var fixPackages = (({ plugins = {} } = {}) => ({ type, name, checks }) => {
    const noFixes = { fixable: [], fixes: {} };
    if (!type) return noFixes;
    if (!plugins[type]) return noFixes;
    if (!plugins[type].fix) return noFixes;

    return plugins[type].fix({ name, checks });
});

function prerelease(config) {
    return check$3(config).extend('unpublishable', ({ packageNames, checks }) => packageNames.filter(name => !checks.packages[name].publishable)).extend('name', ({ unpublishable }) => pickPackage({
        message: 'Which package should we prerelease?',
        packageNames: unpublishable
    })).extend('structure', ({ name }) => getStructures({ packageNames: [name] })[name]).extend('type', ({ name, structure }) => getTypes(config)({
        packageNames: [name],
        structures: { [name]: structure }
    })[name]).extend(fixPackages(config));
}

const command$9 = () => ({
    name: 'prerelease',
    run: prerelease,
    template: template$9,
    effect
});

/* eslint-disable max-lines */

const CI$1 = process.env.CI;

const kli = configure$1({ style: CI$1 ? {} : style });

const cli = kli({
    config: {
        name: 'kikd',
        plugins: [npmPackage, npmCli, vueSite, vueComponent, entyrePackage, luaRock, coreGame],
        plugs: {
            filter: NOOP,
            manifest: NOOP,
            details: NOOBJ,
            build: NOOP,
            postBuild: NOOP,
            config: NOOP,
            preCreate: NOOBJ,
            create: NOOP,
            createTemplate: NOSTR
        }
    },
    default: 'initialize',
    commands: [command$5(), command(), command$6(), command$4(), command$2(), command$3(), command$1(), command$9(), command$7(), command$8()]
});

cli.execute(process.argv.slice(2));
